/** @mainpage Doxygen Example Program
*
* Dies ist ein Beispielprogramm zur Demonstration des Dokumentierungstools Doxygen.<br>
* Das zugehrige Doxygen-Tutorial finden Sie unter <a href="http://www.cypax.net">www.cypax.net</a><br>
* Das Beispielprogramm sortiert Integer-Array mit verschiedenen Sortieralgorithmen.
*
* <img src="application_screenshot.jpg" alt="Screenshot">
*
* @author Philipp Bank
**/


/**
* @file DoxygenExample.java
*
* @brief Diese Datei enthlt die DoxygenExample-Klasse mit der main()-Funktion.
*
* @author Philipp Bank, www.cypax.net\n
*
* @date 18.06.2006 - erste Version
* @date 19.06.2006 - einige Vernderungen, Multi-Threading
*
**/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import sorting.*;


/**
* @class DoxygenExample
* 
* @brief Die DoxygenExample-Klasse erbt von der Frame-Klasse, stellt das grafische Benutzerinterface (GUI) zu Verfgung und enthlt die main()-Funktion.
*
* Diese Klasse erbt von der Frame-Klasse und enthlt die main()-Funktion, welche eine Instanz dieser Klasse erzeugt.<br>
* Das Programm sortiert Integer-Arrays mit:
* <ul>
*   <li> einem <a href="http://de.wikipedia.org/wiki/Selectionsort">Selectionsort-Algorithmus</a></li>
*   <li> einem <a href="http://de.wikipedia.org/wiki/Insertionsort">Insertionsort-Algorithmus</a></li>
*   <li> einem <a href="http://de.wikipedia.org/wiki/Bubblesort">Bubblesort-Algorithmus</a></li>
* </ul>
* Die Algorithmen werden als Threads ausgefhrt und knnen gleichzeitig oder einzeln gestartet werden.<br>
* Um den Sortierprozess zu veranschaulichen, werden die Integer-Arrays als farbige Linien angezeigt.
*/
public class DoxygenExample extends Frame
{

  //Drei Panels, zur Visualisierung der Arrays
  private Panel ssPanel = new Panel(null);
  private Panel isPanel = new Panel(null);
  private Panel bsPanel = new Panel(null);
  
  //drei Label - zeigen den Namen des Algorithmus an
  private Label label1 = new Label();
  private Label label2 = new Label();
  private Label label3 = new Label();
  
  //Ein Panel fr die Buttons
  private Panel buttonPanel = new Panel(null);
  
  private Button startButton = new Button();
  private Button randomizeButton = new Button();
  private Button randStartButton = new Button();
  private Button ssButton = new Button();
  private Button isButton = new Button();
  private Button bsButton = new Button();
  private Button exitButton = new Button();

  //Die TimeLabel zeigen die Laufzeit der Threads an
  private Label ssTimeLabel = new Label();
  private Label isTimeLabel = new Label();
  private Label bsTimeLabel = new Label();

  private int[] ssArray = new int[210];    // das Array fr den Selectionsort-Algorithmus
  private int[] isArray = new int[210];    //           fr den Insertionsort-Algorithmus
  private int[] bsArray = new int[210];    //           fr den Bubblesort-Algorithmus


/**
*
* @brief Der Konstruktor generiert und plaziert die grafischen Komponenten des DoxygenExample-Frames.
*
* @param title - der Titel fr das Frame.
*
*/
  public DoxygenExample(String title) 
  {
    super(title);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) { System.exit(0); }
    });
    
    int frameWidth = 900;
    int frameHeight = 400;
    setSize(frameWidth, frameHeight);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int x = (d.width - getSize().width) / 2;
    int y = (d.height - getSize().height) / 2 ;
    setLocation(x, y);
    Panel cp = new Panel(null);
    add(cp);

    this.setBackground(new Color(240, 240, 240));  

    ssPanel.setBounds(16, 40, 250, 200);
    cp.add(ssPanel);
    isPanel.setBounds(324, 40, 250, 200);
    cp.add(isPanel);
    bsPanel.setBounds(632, 40, 250, 200);
    cp.add(bsPanel);
    label1.setBounds(16, 16, 90, 16);
    label1.setText("Selectionsort");
    label1.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(label1);
    label2.setBounds(324, 16, 90, 16);
    label2.setText("Insertionsort");
    label2.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(label2);
    label3.setBounds(632, 16, 90, 16);
    label3.setText("Bubblesort");
    label3.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(label3);
    buttonPanel.setBounds(16, 280, 860, 81);
    cp.add(buttonPanel);
    startButton.setBounds(140, 48, 110, 25);
    randomizeButton.setBounds(16, 48, 110, 25);
    randStartButton.setBounds(380, 48, 110, 25);
    ssButton.setBounds(16,5,230,25);
    ssButton.setLabel("start");
    startButton.setLabel("start sorting");
    randomizeButton.setLabel("ramdomize");
    randStartButton.setLabel("randomized start");
    buttonPanel.add(randStartButton);
    buttonPanel.add(startButton);
    buttonPanel.add(randomizeButton);
    buttonPanel.add(ssButton);
    buttonPanel.add(isButton);
    buttonPanel.add(bsButton);

    ssButton.setBounds(16,10,220,25);
    ssButton.setLabel("start selectionsort");
    isButton.setBounds(324,10,220,25);
    isButton.setLabel("start insertionsort");
    bsButton.setBounds(632,10,220,25);
    bsButton.setLabel("start bubblesort");

    ssPanel.setBackground(new Color(128,128,128));
    isPanel.setBackground(new Color(128,128,128));
    bsPanel.setBackground(new Color(128,128,128));
    buttonPanel.setBackground(new Color(192,192,192));

    
    randomizeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        randomizeButtonActionPerformed(evt);
      }
    });

    randStartButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        randStartButtonActionPerformed(evt);
      }
    });

    startButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        startButtonActionPerformed(evt);
      }
    });

    ssButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        ssButtonActionPerformed(evt);
      }
    });

    isButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        isButtonActionPerformed(evt);
      }
    });

    bsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        bsButtonActionPerformed(evt);
      }
    });


    exitButton.setBounds(772, 48, 75, 25);
    exitButton.setLabel("exit");
    buttonPanel.add(exitButton);

    exitButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        exitButtonActionPerformed(evt);
      }
    });

    ssTimeLabel.setBounds(16, 256, 120, 16);
    ssTimeLabel.setText("  ");
    ssTimeLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(ssTimeLabel);
    isTimeLabel.setBounds(324, 256, 120, 16);
    isTimeLabel.setText("  ");
    isTimeLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(isTimeLabel);
    bsTimeLabel.setBounds(632, 256, 120, 16);
    bsTimeLabel.setText("  ");
    bsTimeLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(bsTimeLabel);

    setResizable(false);
    setVisible(true);
    
    this.randomizeArrays();  //Arrays mit Zufallszahlen fllen
  }


/**
* @brief Diese Funktion fllt die Arrays mit zufllig erzeugten Zahlen und ruft die Funktion showArray() auf um die Arrays anzuzeigen.
*/
  public void randomizeArrays()
  {
    
    //generiere neue Zufallszahlen
    Random r = new Random(System.currentTimeMillis()); 
   
    for (int i=0; i<210; i++)
    {
      ssArray[i] = 1 + (Math.abs(r.nextInt()) % 180);
      //gleiche Zahlenwerte in allen Arrays
      isArray[i] = ssArray[i];
      bsArray[i] = ssArray[i];
    }

    //Zeige die Zahlen grafisch an
    Paint.showArray(ssArray, ssPanel);
    Paint.showArray(isArray, isPanel);
    Paint.showArray(bsArray, bsPanel);
  }

/**
* @brief Diese Funktion generiert und startet einen neuen Thread fr den Selectionsort-Algorithmus.
*/
  public void startSelectionSort()
  {
    SelectionSort sSort = new SelectionSort();
    sSort.setSettings(this.ssArray, this.ssPanel, this.ssTimeLabel);
    Thread ssThread = new Thread(sSort);  //generiere neuen Thread

    ssThread.start();
    
    //warte bis Thread durchlaufen ist
    try
    {
      ssThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief Diese Funktion generiert und startet einen neuen Thread fr den Insertionsort-Algorithmus.
*/
  public void startInsertionSort()
  {
    InsertionSort iSort = new InsertionSort();
    iSort.setSettings(this.isArray, this.isPanel, this.isTimeLabel);
    Thread isThread = new Thread(iSort);  //generiere neuen Thread


    isThread.start();

    //warte bis Thread durchlaufen ist
    try
    {
      isThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief Diese Funktion generiert und startet einen neuen Thread fr den Bubblesort-Algorithmus.
*/
  public void startBubbleSort()
  {
    BubbleSort bSort = new BubbleSort();
    bSort.setSettings(this.bsArray, this.bsPanel, this.bsTimeLabel);
    Thread bsThread = new Thread(bSort);  //generiere neuen Thread


    bsThread.start();

    //warte bis Thread durchlaufen ist
    try
    {
      bsThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief Diese Funktion generiert und startet alle Sortieralgorithmen gleichzeitig.
*/
  public void sortArrays()
  {
    SelectionSort sSort = new SelectionSort();
    sSort.setSettings(this.ssArray, this.ssPanel, this.ssTimeLabel);
    Thread ssThread = new Thread(sSort);  //generiere neuen Thread

    InsertionSort iSort = new InsertionSort();
    iSort.setSettings(this.isArray, this.isPanel, this.isTimeLabel);
    Thread isThread = new Thread(iSort);  //generiere neuen Thread

    BubbleSort bSort = new BubbleSort();
    bSort.setSettings(this.bsArray, this.bsPanel, this.bsTimeLabel);
    Thread bsThread = new Thread(bSort);  //generiere neuen Thread


    /* Ok, es ist nicht wirklich ein "gleichzeitiger" Start der Threads - wie unten zu sehen ist, ist die Startreihenfolge:
       Selectionsort, Insertionsort und dann Bubblesort.
       Aber sobald ein Thread per start() gestartet wurde, wird sogleich der nchste Thread gestartet und muss nicht warten bis der vorherige
       Thread fertig ist. */
    ssThread.start();
    isThread.start();
    bsThread.start();


    //Warte bis alle Threads fertig sind
    try
    {
      ssThread.join();
    }
    catch (InterruptedException ie)
    {

    }

    try
    {
      isThread.join();
    }
    catch (InterruptedException ie)
    {

    }

    try
    {
      bsThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den ssButton.
* Die Funktion ruft die startSelectionSort()-Funktion auf um einen Selectionsort-Thread zu starten.
*/
  public void ssButtonActionPerformed(ActionEvent evt)
  {
    this.startSelectionSort();
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den isButton.
* Die Funktion ruft die startInsertionSort()-Funktion auf um einen Insertionsort-Thread zu starten.
*/
  public void isButtonActionPerformed(ActionEvent evt)
  {
    this.startInsertionSort();
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den bsButton.
* Die Funktion ruft die startBubbleSort()-Funktion auf um einen Bubblesort-Thread zu starten.
*/
  public void bsButtonActionPerformed(ActionEvent evt)
  {
    this.startBubbleSort();
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den randomizeButton.
* Die Funktion ruft die randomizeArrays()-Funktion auf, welche die Arrays mit Zufallszahlen fllt.
*/
  public void randomizeButtonActionPerformed(ActionEvent evt)
  {
    this.randomizeArrays();
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den randStartButton.
* Diese Funktion macht fast das Gleiche wie randomizeButtonActionPerformed(), aber ruft zudem die sortArrays()-Funktion
* auf um die Arrays mit gleichzeitig laufenden Sortierthreads zu sortiern.
*/
  public void randStartButtonActionPerformed(ActionEvent evt)
  {
    this.randomizeArrays();
    this.sortArrays();
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den startButton.
* Diese Funktion ruft sortArrays() auf ohne(!) zuvor die Arrays mit neuen Zufallszahlen zu fllen.<br>
* Viele Sortieralgorithmen laufen deutlich schneller, wenn das Array bereits vorsortiert ist.
*/
  public void startButtonActionPerformed(ActionEvent evt)
  {
    this.sortArrays();
  }

/**
* @brief Diese Funktion wird ausgefhrt bei Klick auf den exitButton.
* Die Funktion beendet die Anwendung.
*/
  public void exitButtonActionPerformed(ActionEvent evt)
  {
    System.exit(0);
  }

/**
* @brief The main function.
* Der einzige Zweck dieser Funktion ist es eine neue Instanz von DoxygenExample zu erzeugen.
*/
  public static void main(String[] args)
  {
    new DoxygenExample("Selectionsort, Insertionsort and Bubblesort");
  }
}

