/**
* @file Sort.java
*
* @brief Diese Datei enthlt die abstrakte sorting.sort-Klasse.
*
* @author Philipp Bank, www.cypax.net\n
*
* @date 18.06.2006 - erste Version
*
**/

package sorting;

import java.awt.*;

/**
* @class Sort
* @brief Die Sort-Klasse ist die Superklasse fr alle Sortierklassen in diesem Package.
*
* Diese Klasse stellt Funktionen und Eigenschaften zur Verfgung, die von allen Unterklassen benutzt werden.
* Beispielsweise bentigt jeder Sortieralgorithmus eine exchange()-Funktion um zwei Arrayelemente gegenseitig auszutauschen.
* Daher wird diese Funktion einmal in dieser Superklasse implementiert und steht dann allen abgeleiteten Klassen zu Verfgung.
* Andererseits stellt diese Superklasse jedoch lediglich eine leere Sortierfunktion zur Verfgung - die Sortierfunktion muss von
* den Unterklassen mit einem entsprechendem Sortieralgorithmus berschrieben werden
* (daher hat es auch keinen Sinn, von dieser Superklasse eine Instanz zu erzeugen - und daher ist diese Klasse auch abstrakt).
*/
public abstract class Sort implements Runnable
{
  protected long startTime = 0;
  protected long endTime = 0;
  protected long runTime = 0;

  protected int[] theArray;
  protected Panel thePanel;
  protected Label theLabel;

/**
* @brief Die run()-Funktion wird automatisch ausgefhrt, wenn der Thread mit start() aufgerufen wird.
*
* Die run()-Funktion ruft die doSort()-Funktion auf, welche von Unterklassen berschrieben wird.
*/
  public void run()
  {
    this.startTime = System.currentTimeMillis();
    doSort();   // rufe die doSort()-Funktion auf
    this.showArray();
    this.theLabel.setText(this.calcRuntime()+"ms - done!");
    this.theLabel.validate();
  }

/**
* @brief Diese Funktion gibt das Integerarray zurck
*
* Diese Funktion kann benutzt werden um von auerhalb der Klasse Zugriff auf die sortierten Zahlen des Arrays zu erhalten.
*
* @return - gibt theArray zurck
*/
  public int[] getSortedArray()
  {
    return this.theArray;
  }

/**
* @brief Dies ist lediglich eine leere Sortierfunktion (muss von Unterklassen berschrieben werden).
*/
  protected void doSort()
  {
  }


/**
* @brief Diese Funktion vertauscht zwei Arrayelemente.
*
* @param aArray - ein Integerarray
* @param index1 - der Index des ersten Elements
* @param index2 - der Index des zweiten Elements
*/
  protected void exchange(int[] aArray, int index1, int index2)
  {
    int tmp = aArray[index1];
    aArray[index1] = aArray[index2];
    aArray[index2] = tmp;
  }

/**
* @brief Diese Funktion berechnet die Laufzeit des Threads
*
* @return - gibt die Zeit seit Starten des Threads in Millisekunden zurck
*/
  protected long calcRuntime()
  {
    return (System.currentTimeMillis()-startTime);
  }

/**
* @brief Diese Funktion veranschaulicht Arraywerte durch grafische Ausgabe der Werte als vertikale, farbige Linien auf einem Panel.
*
* @note: Dise Funktion setzt voraus, dass die setSettings()-Funktion zuvor aufgerufen wurde.
*/
  public void showArray()
  {
    Graphics g = thePanel.getGraphics();
    Color lc = new Color(0,255,0);
    Color bc = new Color(128,128,128);
    for (int i=0; i<210; i++)
    {
      g.setColor(lc);
      g.drawLine((i+20),190,(i+20),(190-theArray[i]));
      g.setColor(bc);
      g.drawLine((i+20),(190-theArray[i]),(i+20),10);
    }
    thePanel.validate();
  }

/**
* @brief Diese Funktion wird benutzt um die Attribute theArray, thePanel und theLabel zu setzen.
*
* @param aArray - ein zu sortierendes Integerarray
* @param aPanel - ein Panel, auf dem die grafische Darstellung des Arrays whrend des Sortierens gezeichnet werden soll
* @param aLabel - ein Label, das die Laufzeit des Threads anzeigen soll
*/
  public void setSettings(int[] aArray, Panel aPanel, Label aLabel)
  {
    this.theArray = aArray;
    this.thePanel = aPanel;
    this.theLabel = aLabel;
  }
  
}

